// Pingus - A free Lemmings clone
// Copyright (C) 1999 Ingo Ruhnke <grumbel@gmx.de>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <ClanLib/clanlayer2.h>

#include "PingusSound.hh"
#include "globals.hh"
#include "plf_datatypes.hh"
#include "Pingu.hh"
#include "ButtonPanel.hh"
#include "actions/pingu_actions.hh"

CL_Surface* ButtonPanel::button_cap;

ButtonPanel::ButtonPanel(PLF* plf)
{
  need_redraw = true;
  pressed_button = 2;

  vector<button_data> buttons = plf->get_buttons();

  for(unsigned int i=0; i < buttons.size(); ++i) {
    button.push_back(new ActionButton(38 * i,
				      CL_Display::get_height() - 56, buttons[i].name));
  }

  armageddon_sur = CL_Res_Surface::load("Buttons/armageddon", *resource());

  button_cap      = CL_Res_Surface::load("Buttons/button_cap", *resource());
}

ButtonPanel::~ButtonPanel()
{
}

void
ButtonPanel::let_move() 
{
  button[pressed_button]->let_move();
  
  if (CL_Mouse::left_pressed())
    {
      int mouse_x = CL_Mouse::get_x();
      int mouse_y = CL_Mouse::get_y();
      int temp_button;
      
      if (mouse_y > CL_Display::get_height() - 56
	  && mouse_x < ((button.size() + 1) * 38))
	{
	  temp_button = mouse_x / 38;
	  if (temp_button == button.size()) {
	    world->armageddon();
	  } else {
	    pressed_button = temp_button;
	  }
	}
    }
}
/*
PinguAction*
ButtonPanel::get_action()
{
  return button[pressed_button]->get_action();
}*/

string
ButtonPanel::get_action_name()
{
  return button[pressed_button]->get_action_name();
}

void 
ButtonPanel::draw() 
{
  // BUG: some optimizantions could be usefull
  need_redraw = true;
  if (need_redraw) {
    // draw the buttons
    for(unsigned int i=0; i < button.size(); ++i) {
      button[i]->draw();
    }
    armageddon_sur->put_screen(38 * button.size(), CL_Display::get_height() - 56);
    need_redraw = false;
  }

  // draw the capture rectangle for the active button
  button_cap->put_screen(pressed_button*38, CL_Display::get_height() - 56);
}

void
ButtonPanel::set_world(World* w)
{
  world = w;
}

/* EOF */
