// Pingus - A free Lemmings clone
// Copyright (C) 1999 Ingo Ruhnke <grumbel@gmx.de>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <dirent.h>
#include <fnmatch.h>
#include <algorithm>
#include <ClanLib/clanlib.h>
#include "globals.hh"
#include "FileSelector.hh"

FileSelector::FileSelector()
{
  if (verbose > 1)
    cout << "Construction FileSelector" << endl;
  font = CL_Res_Font::load("Fonts/courier",*resource());
  font_h = CL_Res_Font::load("Fonts/courier_h",*resource());
  
  up = CL_Res_Surface::load("Buttons/up", *resource());
  down = CL_Res_Surface::load("Buttons/down", *resource());

  bg = CL_Res_Surface::load("Textures/stone", *resource());

  file_index = -1;
  file_offset = 0;

  down_x = 320 - (up->get_width()/2);
  down_y = 320;
  up_x = 320 - (up->get_width()/2);
  up_y = 100;
}

string
FileSelector::select(string pathname, string pattern)
{
  string filename;

  CL_Display::sync_buffers();

  FileSelector::readdir(pathname, pattern);
  FileSelector::draw();

  filename = FileSelector::select_file();

  if (filename == "")
    return "";

  return pathname + filename;
}

string
FileSelector::select_file(void)
{
  int temp_index = -1;

  while(true) {
    if (CL_Mouse::left_pressed()) {
      while(CL_Mouse::left_pressed()) 
	CL_System::keep_alive();
      
      if (file_index != -1)
	break;
      
      if (up_x < CL_Mouse::get_x()
	  && up_x + up->get_width() > CL_Mouse::get_x()
	  && up_y < CL_Mouse::get_y()
	  && up_y + up->get_height() > CL_Mouse::get_y()) {
	--file_offset;
      } else if (down_x < CL_Mouse::get_x()
		 && down_x + down->get_width() > CL_Mouse::get_x()
		 && down_y < CL_Mouse::get_y()
		 && down_y + down->get_height() > CL_Mouse::get_y()) {
	++file_offset;
      }
      
      if (file_offset < 0) {
	file_offset = 0;
      } else if (file_offset + 5 > directory.size()) {
	file_offset = directory.size() - 5;
      }
      draw();
    }
    CL_System::keep_alive();

    if (CL_Keyboard::get_keycode(CL_Keyboard::KEY_ESCAPE))
      return "";

    // If something as changed update the screen
    if (temp_index != file_index) {
      draw(file_index);
      temp_index = file_index;
    }

    // Get the selected item
    file_index = (CL_Mouse::get_y() - 200) / 20 + file_offset;

    if (file_index < file_offset) {
      file_index = -1;
    } else if (file_index >= file_offset + 5) {
      file_index = -1;
    }

    // Check if is in the size of the vector    
    if (file_index < 0)
      file_index = - 1;
    else if (file_index >= directory.size()) 
      file_index = -1;
    
    // Check if the mouse is on the item on not beneath them
    if (file_index != -1 
	&& (CL_Mouse::get_x() < ((CL_Display::get_width() / 2)
				-  (font->get_text_width(directory[file_index].c_str()) /2 ))
	|| CL_Mouse::get_x() > ((CL_Display::get_width() / 2)
				+  (font->get_text_width(directory[file_index].c_str()) / 2))))
      {
	file_index = -1;
      }
  }
  return directory[file_index];
}

void
FileSelector::readdir(string pathname, string pattern)
{
  DIR* dp;
  dirent* de;

  dp = opendir(pathname.c_str());
  
  if (!dp) {
    perror(pathname.c_str());
    exit(EXIT_FAILURE);
  }

  while ((de = ::readdir(dp)) != NULL) {
    if (fnmatch(pattern.c_str(), de->d_name, FNM_PATHNAME) == 0)
      directory.push_back(de->d_name);
  } // BUG, errors aren't checked
  closedir(dp);

  // Sort the directory list
  sort(directory.begin(), directory.end());
}

// BUG: The file selection object needs some work, it doesn't handle
// more than five or sixe files correct, because it has no support for 
// scrolling at the moment.
void
FileSelector::draw(int highlight)
{
  CL_Display::clear_display();

  for(int y = 0; y < CL_Display::get_height(); y += bg->get_height()) {
    for(int x = 0; x < CL_Display::get_width(); x += bg->get_width()) {
      bg->put_screen(x, y);
    }
  }

  CL_Display::fill_rect(250, 180, 390, 320, 0.0, 0.0, 0.0, 0.5);

  for(unsigned int i=file_offset; i < (file_offset + 5) && i < directory.size(); ++i) {
    if (file_index == i)
      font_h->print_center(320,20*(i-file_offset) + 200, directory[i].c_str());
    else
      font->print_center(320,20*(i-file_offset) + 200, directory[i].c_str());
  }
  up->put_screen(up_x, up_y);
  down->put_screen(down_x, down_y);

  CL_Display::flip_display();
}

/* EOF */
