// Pingus - A free Lemmings clone
// Copyright (C) 1999 Ingo Ruhnke <grumbel@gmx.de>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <cstdio>

#include "PingusError.hh"
#include "globals.hh"
#include "PinguTileMap.hh"

PinguTileMap::PinguTileMap(PLF* data, CL_ResourceManager* resource)
{
  switch(data->get_fg().type) {
    case ResDescriptor::RESOURCE :
      cout << "PinguTileMap: ResType RESOURCE is not implemented, sorry" << endl;
      exit(EXIT_FAILURE);
      break;
    case ResDescriptor::FILE:
      load(data->get_fg().res_name.c_str(), resource);
      break;
  }
}

PinguTileMap::~PinguTileMap()
{
}

void
PinguTileMap::mark_dirty(int x, int y, int w, int h)
{
  return;
  // This border checker is not very flexible
  if (x > 800 || x < 0 || y < 0 | y > 600)
      return;
  
  for(int j=y;  j <= y+h+32; j+=32)
    for(int i=x; i <= x+w+32; i+=32)
      map[j/32][i/32].set_redraw(true);
}

int
PinguTileMap::load(const char* filename, CL_ResourceManager* resource)
{
  int tmp;

  ifstream in(filename);

  if (!in) {
    throw PingusError(string("Couldn't open file: ") + filename);
  }

  in >> width;
  in >> height;

  map.resize(height);
  for(unsigned int i=0; i < map.size(); ++i) {
    map[i].resize(width);
  }
  
  for(unsigned int y=0; y < map.size(); ++y) {
    for(unsigned int x=0; x < map[y].size(); ++x) {
      in >> tmp;
      map[y][x].load(tmp, resource);
    }
  }
  return 0;
}

int
PinguTileMap::load(ifstream* in)
{
  int w,h;

  (*in) >> w >> h;

  map.resize(h);
  
  for(unsigned int i=0; i < map.size(); ++i)
    map[i].resize(i);

  return true;
}

void
PinguTileMap::draw_offset(int of_x, int of_y, double s = 1.0)
{
  if (s == 1.0) {
    for(int y=0; y < static_cast<int>(map.size()); ++y) {
      for(int x=0; x < static_cast<int>(map[y].size()); ++x) {
	if (x >= 0 && x < width && y >=0 && y < height) 
	  map[y][x].draw(x*32 + of_x, y*32 + of_y);
      }
    }
  } else {
    for(int y=0; y < static_cast<int>(map.size()); ++y) {
      for(int x=0; x < static_cast<int>(map[y].size()); ++x) {
	if (x >= 0 && x < width && y >=0 && y < height) 
	  map[y][x].draw(x*32 + of_x, 
			 y*32 + of_y,
			 s
			 );
      }
    }
  }
}

void
PinguTileMap::start_anim()
{
}

void
PinguTileMap::stop_anim()
{
}

int
PinguTileMap::get_width()
{
  return width*32;
}

int
PinguTileMap::get_height()
{
  return height*32;
}

/* EOF */
